/**
 * @file        邮件服务配置
 * @author      hotok
 * @date        2025-06-29
 * @lastEditor  hotok
 * @lastEditTime 2025-06-29
 * @description 读取环境变量并导出邮件服务配置
 */

/**
 * SMTP服务器配置
 * @property {string} host     - SMTP服务器主机地址
 * @property {number} port     - SMTP服务器端口号
 * @property {boolean} secure  - 是否使用SSL/TLS
 * @property {object} auth     - 认证信息
 */
export const smtpConfig = {
    /** SMTP服务器主机地址 */
    host: process.env.SMTP_HOST || 'smtp.qq.com',
    /** SMTP服务器端口号 */
    port: Number(process.env.SMTP_PORT) || 587,
    /** 是否使用SSL/TLS */
    secure: process.env.SMTP_SECURE === 'true' || false,
    /** 认证信息 */
    auth: {
        user: process.env.SMTP_USER || '',
        pass: process.env.SMTP_PASS || '',
    },
    /** 连接超时时间（毫秒） */
    connectionTimeout: Number(process.env.SMTP_TIMEOUT) || 60000,
    /** 问候超时时间（毫秒） */
    greetingTimeout: Number(process.env.SMTP_GREETING_TIMEOUT) || 30000,
    /** 套接字超时时间（毫秒） */
    socketTimeout: Number(process.env.SMTP_SOCKET_TIMEOUT) || 60000,
};

/**
 * 邮件基础配置
 * @property {string} from     - 发件人信息
 * @property {string} replyTo  - 回复邮箱
 * @property {string} charset  - 字符编码
 */
export const emailConfig = {
    /** 发件人信息 - QQ邮箱要求From地址必须与SMTP用户名一致 */
    from: process.env.SMTP_FROM_EMAIL || process.env.SMTP_USER || '',
    /** 发件人名称 */
    fromName: process.env.SMTP_FROM_NAME || '星撰系统',
    /** 回复邮箱 */
    replyTo: process.env.EMAIL_REPLY_TO || process.env.SMTP_USER || '',
    /** 字符编码 */
    charset: 'utf-8',
    /** 邮件优先级 */
    priority: 'normal' as const,
};

/**
 * 邮件模板配置
 * @property {object} activation      - 账号激活邮件模板
 * @property {object} passwordReset   - 密码重置邮件模板
 * @property {object} welcome         - 欢迎邮件模板
 * @property {object} notification    - 通知邮件模板
 */
export const emailTemplates = {
    /** 账号激活邮件模板 */
    activation: {
        subject: '请激活您的账户 - 星撰玉衡',
        template: 'activation',
        expireTime: 24 * 60 * 60 * 1000, // 24小时
    },
    /** 密码重置邮件模板 */
    passwordReset: {
        subject: '重置您的密码 - 星撰玉衡',
        template: 'password-reset',
        expireTime: 30 * 60 * 1000, // 30分钟
    },
    /** 欢迎邮件模板 */
    welcome: {
        subject: '欢迎加入星撰玉衡',
        template: 'welcome',
    },
    /** 通知邮件模板 */
    notification: {
        subject: '系统通知 - 星撰玉衡',
        template: 'notification',
    },
    /** 密码修改通知模板 */
    passwordChanged: {
        subject: '密码已修改 - 星撰玉衡',
        template: 'password-changed',
    },
};

/**
 * 邮件发送选项配置
 * @property {number} retryAttempts - 重试次数
 * @property {number} retryDelay    - 重试延迟时间（毫秒）
 * @property {boolean} enableQueue - 是否启用队列发送
 */
export const emailOptions = {
    /** 重试次数 */
    retryAttempts: Number(process.env.EMAIL_RETRY_ATTEMPTS) || 3,
    /** 重试延迟时间（毫秒） */
    retryDelay: Number(process.env.EMAIL_RETRY_DELAY) || 5000,
    /** 是否启用队列发送 */
    enableQueue: process.env.EMAIL_ENABLE_QUEUE === 'true' || false,
    /** 单次发送最大收件人数量 */
    maxRecipients: Number(process.env.EMAIL_MAX_RECIPIENTS) || 50,
    /** 发送频率限制（邮件/分钟） */
    rateLimit: Number(process.env.EMAIL_RATE_LIMIT) || 100,
};

/**
 * 验证邮件配置
 * @returns {boolean} 配置是否有效
 */
export const validateEmailConfig = (): boolean => {
    const { host, auth } = smtpConfig;
    return !!(host && auth.user && auth.pass);
};

/**
 * 获取完整的邮件配置
 * @returns 完整的邮件服务配置对象
 */
export const getEmailConfig = () => ({
    smtp: smtpConfig,
    email: emailConfig,
    templates: emailTemplates,
    options: emailOptions,
}); 