# 邮件服务插件

邮件服务插件为 Elysia 应用提供完整的邮件发送功能，包括模板邮件、重试机制、健康检查等。

## 特性

- 🚀 **易于使用**: 简单的 API 接口，支持链式调用
- 📧 **模板支持**: 内置 5 种邮件模板（激活、密码重置、欢迎、通知、密码修改）
- 🔄 **重试机制**: 自动重试失败的邮件发送
- 📊 **健康检查**: 提供 SMTP 连接状态监控
- 🎨 **美观模板**: 响应式 HTML 邮件模板
- 🔒 **类型安全**: 完整的 TypeScript 类型支持
- 📝 **详细日志**: 完整的邮件发送日志记录

## 安装和配置

### 1. 环境变量配置

在 `.env` 文件中配置邮件服务参数：

```env
# SMTP 服务器配置
SMTP_HOST=smtp.qq.com
SMTP_PORT=587
SMTP_SECURE=false
SMTP_USER=your-email@qq.com
SMTP_PASS=your-password

# 邮件基础配置
EMAIL_FROM="星撰系统 <your-email@qq.com>"
EMAIL_REPLY_TO=your-email@qq.com

# 重试配置
EMAIL_RETRY_ATTEMPTS=3
EMAIL_RETRY_DELAY=5000
```

### 2. 插件注册

在 Elysia 应用中注册邮件插件：

```typescript
import { Elysia } from 'elysia';
import { emailPlugin } from '@/plugins/email/email.plugins';

const app = new Elysia()
    .use(emailPlugin)
    // ... 其他插件
```

## 使用方法

### 基础邮件发送

```typescript
app.post('/send-email', async ({ emailService }) => {
    const result = await emailService.sendEmail({
        to: 'user@example.com',
        subject: '测试邮件',
        html: '<h1>Hello World!</h1>',
        text: 'Hello World!',
    });

    return { success: result.success, messageId: result.messageId };
});
```

### 模板邮件发送

#### 1. 用户激活邮件

```typescript
app.post('/send-activation', async ({ emailService }) => {
    return await emailService.sendActivationEmail({
        to: 'user@example.com',
        username: 'john_doe',
        nickname: 'John',
        activationUrl: 'https://example.com/activate?token=xxx',
        activationCode: '123456',
        expireTime: '24小时',
    });
});
```

#### 2. 密码重置邮件

```typescript
app.post('/send-reset', async ({ emailService }) => {
    return await emailService.sendPasswordResetEmail({
        to: 'user@example.com',
        username: 'john_doe',
        nickname: 'John',
        resetUrl: 'https://example.com/reset?token=xxx',
        resetCode: '123456',
        expireTime: '30分钟',
    });
});
```

#### 3. 欢迎邮件

```typescript
app.post('/send-welcome', async ({ emailService }) => {
    return await emailService.sendWelcomeEmail({
        to: 'user@example.com',
        username: 'john_doe',
        nickname: 'John',
    });
});
```

#### 4. 密码修改通知

```typescript
app.post('/send-password-changed', async ({ emailService }) => {
    return await emailService.sendPasswordChangedEmail({
        to: 'user@example.com',
        username: 'john_doe',
        nickname: 'John',
    });
});
```

#### 5. 系统通知邮件

```typescript
app.post('/send-notification', async ({ emailService }) => {
    return await emailService.sendNotificationEmail({
        to: ['user1@example.com', 'user2@example.com'],
        username: 'john_doe',
        message: '您有新的系统消息，请及时查看。',
        subject: '重要通知',
    });
});
```

### 服务状态检查

```typescript
app.get('/email/status', ({ emailService }) => {
    return {
        status: emailService.getStatus(),
        initialized: emailService.isInitialized(),
    };
});

app.get('/email/health', async ({ emailService }) => {
    return await emailService.healthCheck();
});
```

## API 参考

### emailService.sendEmail(options)

发送自定义邮件。

**参数:**
- `options.to`: 收件人邮箱（字符串或数组）
- `options.subject`: 邮件主题
- `options.html`: HTML 内容
- `options.text`: 纯文本内容
- `options.cc`: 抄送邮箱（可选）
- `options.bcc`: 密送邮箱（可选）
- `options.attachments`: 附件列表（可选）
- `options.priority`: 邮件优先级（可选）

### emailService.sendTemplateEmail(options)

发送模板邮件。

**参数:**
- `options.to`: 收件人邮箱
- `options.template`: 模板类型
- `options.params`: 模板参数
- `options.subject`: 自定义主题（可选）
- `options.priority`: 邮件优先级（可选）

### 便捷方法

- `sendActivationEmail()`: 发送激活邮件
- `sendPasswordResetEmail()`: 发送密码重置邮件
- `sendWelcomeEmail()`: 发送欢迎邮件
- `sendPasswordChangedEmail()`: 发送密码修改通知
- `sendNotificationEmail()`: 发送系统通知

### 状态检查

- `getStatus()`: 获取服务状态
- `healthCheck()`: 执行健康检查
- `isInitialized()`: 检查是否已初始化

## 错误处理

插件内置了完善的错误处理机制：

- **配置验证**: 启动时验证 SMTP 配置
- **连接检查**: 健康检查验证 SMTP 连接
- **重试机制**: 自动重试失败的邮件发送
- **优雅降级**: 邮件服务故障不影响应用启动

## 日志记录

所有邮件操作都会记录详细日志：

- 邮件发送成功/失败
- 重试次数和原因
- 响应时间
- 错误详情

## 注意事项

1. **SMTP 认证**: 确保 SMTP 用户名和密码正确
2. **端口配置**: 根据邮件提供商配置正确的端口和 SSL 设置
3. **发送限制**: 注意邮件提供商的发送频率限制
4. **模板自定义**: 可以通过修改服务类来自定义邮件模板
5. **安全性**: 避免在邮件内容中包含敏感信息 