/**
 * @file        JWT服务类
 * @author      hotok
 * @date        2025-06-29
 * @lastEditor  hotok
 * @lastEditTime 2025-06-29
 * @description 提供类型安全的JWT生成、验证和管理功能
 */

import { jwtConfig } from '@/config/jwt.config';
import { Logger } from '@/plugins/logger/logger.service';
import type { 
    JwtUserType, 
    JwtPayloadType, 
    JwtSignOptionsType,
} from '@/type/jwt.type';
import type { UserInfoType } from '@/modules/example/example.schema';

/**
 * JWT服务类
 * @description 提供JWT Token的生成、验证、刷新等功能
 */
export class JwtService {
    /**
     * 生成JWT Token
     * @param userInfo 完整的用户信息
     * @param options 可选的JWT配置
     * @returns Promise<string> JWT Token字符串
     * @modification hotok 2025-06-29 初始实现JWT生成功能
     */
    async generateToken(userInfo: UserInfoType, options?: Partial<JwtSignOptionsType>): Promise<string> {
        try {
            // 从完整用户信息提取JWT载荷所需的字段
            const jwtUser: JwtUserType = {
                userId: userInfo.id,
                username: userInfo.username,
                email: userInfo.email,
                nickname: userInfo.nickname,
                status: userInfo.status,
                role: options?.user?.role, // 如果有传入角色信息
            };

                         // 构建JWT载荷
             const payload: Omit<JwtPayloadType, 'iat' | 'exp'> = {
                 ...jwtUser,
                 sub: userInfo.id.toString(),
                 iss: options?.issuer || 'elysia-api',
                 aud: options?.audience || 'web-client',
             };

             // 注意：实际的token生成需要使用Elysia的jwt实例
             // 这里提供接口定义，具体实现需要在controller中使用jwt.sign
             const token = 'generated-token-placeholder';

            Logger.info(`JWT Token生成成功，用户ID: ${userInfo.id}, 用户名: ${userInfo.username}`);
            return token;

        } catch (error) {
            Logger.error(new Error(`JWT Token生成失败: ${error}`));
            throw new Error('Token生成失败');
        }
    }

    /**
     * 验证JWT Token
     * @param token JWT Token字符串
     * @returns Promise<JwtPayloadType | null> 验证成功返回载荷，失败返回null
     * @modification hotok 2025-06-29 初始实现JWT验证功能
     */
         async verifyToken(token: string): Promise<JwtPayloadType | null> {
         try {
             // 注意：实际的token验证需要使用Elysia的jwt实例
             // 这里提供接口定义，具体实现需要在controller中使用jwt.verify
             const payload = null as any as JwtPayloadType;
            
            if (!payload || !payload.userId) {
                Logger.warn(`JWT Token验证失败：载荷无效`);
                return null;
            }

            // 检查Token是否过期
            const now = Math.floor(Date.now() / 1000);
            if (payload.exp && payload.exp < now) {
                Logger.warn(`JWT Token已过期，用户ID: ${payload.userId}`);
                return null;
            }

            Logger.info(`JWT Token验证成功，用户ID: ${payload.userId}`);
            return payload;

        } catch (error) {
            Logger.warn(`JWT Token验证失败: ${error}`);
            return null;
        }
    }

    /**
     * 刷新JWT Token
     * @param oldToken 旧的JWT Token
     * @param userInfo 最新的用户信息
     * @returns Promise<string | null> 新的JWT Token，失败返回null
     * @modification hotok 2025-06-29 初始实现JWT刷新功能
     */
    async refreshToken(oldToken: string, userInfo: UserInfoType): Promise<string | null> {
        try {
            // 验证旧Token
            const oldPayload = await this.verifyToken(oldToken);
            if (!oldPayload) {
                                 Logger.warn('Token刷新失败：旧Token无效');
                return null;
            }

            // 生成新Token
            const newToken = await this.generateToken(userInfo);
            
            Logger.info(`JWT Token刷新成功，用户ID: ${userInfo.id}`);
            return newToken;

        } catch (error) {
            Logger.error(new Error(`JWT Token刷新失败: ${error}`));
            return null;
        }
    }

    /**
     * 提取JWT载荷中的用户信息
     * @param payload JWT载荷
     * @returns JwtUserType 用户信息
     * @modification hotok 2025-06-29 添加用户信息提取功能
     */
    extractUserFromPayload(payload: JwtPayloadType): JwtUserType {
        return {
            userId: payload.userId,
            username: payload.username,
            email: payload.email,
            nickname: payload.nickname,
            status: payload.status,
            role: payload.role,
        };
    }

    /**
     * 检查用户Token是否即将过期
     * @param payload JWT载荷
     * @param thresholdMinutes 阈值分钟数（默认30分钟）
     * @returns boolean 是否即将过期
     * @modification hotok 2025-06-29 添加Token过期检查功能
     */
    isTokenExpiringSoon(payload: JwtPayloadType, thresholdMinutes: number = 30): boolean {
        if (!payload.exp) return false;
        
        const now = Math.floor(Date.now() / 1000);
        const threshold = thresholdMinutes * 60; // 转换为秒
        
        return (payload.exp - now) <= threshold;
    }

    /**
     * 获取Token剩余有效时间
     * @param payload JWT载荷
     * @returns number 剩余秒数，-1表示已过期
     * @modification hotok 2025-06-29 添加Token时间计算功能
     */
    getTokenRemainingTime(payload: JwtPayloadType): number {
        if (!payload.exp) return -1;
        
        const now = Math.floor(Date.now() / 1000);
        const remaining = payload.exp - now;
        
        return remaining > 0 ? remaining : -1;
    }
}

/**
 * 导出JWT服务实例
 */
export const jwtService = new JwtService(); 