/**
 * @file        HTTP请求日志插件
 * @author      hotok
 * @date        2024-01-15
 * @lastEditor  hotok
 * @lastEditTime 2024-01-15
 * @description 记录HTTP请求和响应的详细信息，包括请求时间、响应时间、客户端信息等
 */

import { Elysia } from 'elysia';
import { browserInfo } from '@/utils/deviceInfo';
import { getRandomBackgroundColor } from '@/utils/randomChalk';
import Logger, { getResponseSize } from '@/plugins/logger/logger.service';

/**
 * HTTP请求日志插件
 * @description 为Elysia应用添加请求和响应日志功能
 * @param app Elysia应用实例
 * @returns 配置了日志功能的Elysia应用
 * @example
 * const app = new Elysia().use(loggerPlugin)
 */
export const loggerPlugin = (app: Elysia) =>
    app
        /** 注册请求开始时间到应用状态，用于计算响应时间 */
        .state('requestStart', null as [number, number] | null)
        .state('color', null as string | null)
        /** 请求拦截器 - 记录请求信息 */
        .onRequest(({ store, request, server, path,}) => {
            /** 记录请求开始时间 */
            store.requestStart = process.hrtime();
            /** 获取客户端IP信息 */
            const clientIP = server?.requestIP(request);
            store.color = getRandomBackgroundColor()(' ');
            /** 记录请求日志 */
            Logger.http({
                type: 'request',
                method: request.method,
                path,
                color: store.color,
                ip: `${clientIP?.family}: ${clientIP?.address}:${clientIP?.port}`,
                browser: browserInfo(request.headers.get('user-agent')),
            });
        })
        /** 响应拦截器 - 记录响应信息 */
        .onAfterResponse(({ store, request, set, response, path }) => {
            /** 计算请求处理时间 */
            const [seconds, nanoseconds] = process.hrtime(store.requestStart);
            const duration = seconds * 1000 + nanoseconds / 1000000;

            /** 记录响应日志 */
            Logger.http({
                type: 'response',
                method: request.method,
                path,
                color: store.color,
                statusCode: set.status || 200,
                requestTime: `${duration.toFixed(2)}ms`,
                responseSize: getResponseSize(response),
            });
        });
