 /**
 * @file        邮件发送Demo
 * @author      hotok
 * @date        2025-06-29
 * @description 简单的邮件发送测试demo，用于验证邮件服务是否正常工作
 */

import { 
    initializeEmailService,
    sendEmail,
    sendTemplateEmail,
    checkEmailServiceHealth,
    closeEmailService
} from '@/plugins/email/email.service';
import { validateEmailConfig } from '@/config/email.config';
import type { EmailSendOptions, EmailTemplateSendOptions } from '@/type/email.type';

// 邮件发送Demo类
class EmailDemo {
    private initialized = false;

    /**
     * 初始化邮件服务
     */
    async init(): Promise<boolean> {
        try {
            console.log('🚀 正在初始化邮件服务...');
            
            // 检查配置
            const isConfigValid = validateEmailConfig();
            if (!isConfigValid) {
                console.log('❌ 邮件配置无效，请检查环境变量配置');
                return false;
            }

            // 初始化服务
            await initializeEmailService();
            this.initialized = true;
            
            console.log('✅ 邮件服务初始化成功');
            return true;
        } catch (error) {
            console.log('❌ 邮件服务初始化失败:', error);
            return false;
        }
    }

    /**
     * 健康检查
     */
    async healthCheck(): Promise<void> {
        console.log('\n🔍 执行健康检查...');
        try {
            const health = await checkEmailServiceHealth();
            console.log(`📊 健康状态: ${health.status}`);
            console.log(`⏱️  响应时间: ${health.responseTime}ms`);
            console.log(`🔗 服务状态: ${health.serviceStatus}`);
        } catch (error) {
            console.log('❌ 健康检查失败:', error);
        }
    }

    /**
     * 发送简单测试邮件
     */
    async sendSimpleEmail(to: string): Promise<void> {
        console.log(`\n📧 发送简单测试邮件到: ${to}`);
        
        const emailOptions: EmailSendOptions = {
            to: to,
            subject: '邮件服务测试 - 简单邮件',
            text: '这是一封测试邮件的纯文本内容。如果您收到这封邮件，说明邮件服务工作正常！',
            html: `
                <div style="font-family: Arial, sans-serif; max-width: 600px; margin: 0 auto;">
                    <h2 style="color: #2563eb;">🎉 邮件服务测试成功！</h2>
                    <p>这是一封<strong>HTML格式</strong>的测试邮件。</p>
                    <div style="background: #f3f4f6; padding: 20px; border-radius: 8px; margin: 20px 0;">
                        <p><strong>测试信息：</strong></p>
                        <ul>
                            <li>发送时间: ${new Date().toLocaleString('zh-CN')}</li>
                            <li>邮件类型: HTML格式</li>
                            <li>服务状态: 正常运行</li>
                        </ul>
                    </div>
                    <p style="color: #6b7280;">如果您收到这封邮件，说明邮件发送服务配置正确！</p>
                </div>
            `,
        };

        try {
            const result = await sendEmail(emailOptions);
            
            if (result.success) {
                console.log('✅ 邮件发送成功！');
                console.log(`📮 消息ID: ${result.messageId}`);
                console.log(`📬 接收方: ${result.accepted?.join(', ')}`);
            } else {
                console.log('❌ 邮件发送失败');
                console.log(`💥 错误信息: ${result.error}`);
                if (result.rejected && result.rejected.length > 0) {
                    console.log(`🚫 被拒绝的邮箱: ${result.rejected.join(', ')}`);
                }
            }
        } catch (error) {
            console.log('❌ 发送过程中出现异常:', error);
        }
    }

    /**
     * 发送模板邮件测试
     */
    async sendTemplateEmail(to: string): Promise<void> {
        console.log(`\n📧 发送模板邮件到: ${to}`);
        
        const templateOptions: EmailTemplateSendOptions = {
            to: to,
            template: 'welcome',
            params: {
                username: 'demo_user',
                nickname: '测试用户',
                email: to,
            },
        };

        try {
            const result = await sendTemplateEmail(templateOptions);
            
            if (result.success) {
                console.log('✅ 模板邮件发送成功！');
                console.log(`📮 消息ID: ${result.messageId}`);
                console.log(`📬 接收方: ${result.accepted?.join(', ')}`);
            } else {
                console.log('❌ 模板邮件发送失败');
                console.log(`💥 错误信息: ${result.error}`);
            }
        } catch (error) {
            console.log('❌ 发送过程中出现异常:', error);
        }
    }

    /**
     * 关闭服务
     */
    async close(): Promise<void> {
        if (this.initialized) {
            await closeEmailService();
            console.log('\n🔚 邮件服务已关闭');
        }
    }
}

// 主函数
async function main() {
    console.log('='.repeat(50));
    console.log('📧 邮件发送Demo - 验证邮件服务功能');
    console.log('='.repeat(50));

    const demo = new EmailDemo();
    
    try {
        // 初始化
        const initSuccess = await demo.init();
        if (!initSuccess) {
            console.log('\n💡 请检查以下环境变量配置：');
            console.log('- SMTP_HOST: SMTP服务器地址');
            console.log('- SMTP_PORT: SMTP端口');
            console.log('- SMTP_USER: 邮箱账号');
            console.log('- SMTP_PASS: 邮箱密码/授权码');
            console.log('- SMTP_FROM_NAME: 发件人名称');
            console.log('- SMTP_FROM_EMAIL: 发件人邮箱');
            return;
        }

        // 健康检查
        await demo.healthCheck();

        // 询问收件人邮箱
        const testEmail = process.argv[2] || 'test@example.com';
        console.log(`\n🎯 测试邮箱: ${testEmail}`);
        
        if (testEmail === 'test@example.com') {
            console.log('💡 提示: 可以通过参数指定邮箱地址');
            console.log('   示例: bun run src/tests/demo/emailDemo.ts your@email.com');
        }

        // 发送测试邮件
        await demo.sendSimpleEmail(testEmail);
        
        // 等待一下再发送模板邮件
        console.log('\n⏳ 等待2秒后发送模板邮件...');
        await new Promise(resolve => setTimeout(resolve, 2000));
        
        await demo.sendTemplateEmail(testEmail);

        console.log('\n' + '='.repeat(50));
        console.log('✨ Demo执行完成！请检查您的邮箱收件箱');
        console.log('📬 如果没收到邮件，请检查垃圾邮件文件夹');
        console.log('='.repeat(50));

    } catch (error) {
        console.log('💥 Demo执行过程中出现错误:', error);
    } finally {
        // 关闭服务
        await demo.close();
    }
}

// 如果直接运行此文件
if (import.meta.main) {
    main().catch(console.error);
}

export { EmailDemo };