/**
 * @file        健康检查功能测试
 * @author      hotok
 * @date        2025-06-28
 * @lastEditor  hotok
 * @lastEditTime 2025-06-28
 * @description 测试健康检查接口和服务，包括系统状态、依赖检查、响应格式等
 */

import { describe, it, expect, beforeAll, afterAll } from 'vitest';
import { setTimeout } from 'node:timers';
import { app } from '@/app';

describe('健康检查接口测试', () => {
    beforeAll(async () => {
        // 等待应用启动
        await new Promise(resolve => setTimeout(resolve, 1000));
    });

    describe('GET /api/health', () => {
        it('应该返回基本健康状态', async () => {
            const res = await app.fetch(
                new Request('http://localhost/api/health', {
                    method: 'GET',
                    headers: { 'Content-Type': 'application/json' },
                }),
            );
            
            const body = (await res.json()) as any;
            
            expect(res.status).toBe(200);
            expect(body.code).toBeTypeOf('number');
            expect(body.message).toBeTypeOf('string');
            expect(body.data).toBeTypeOf('object');
            
            // 检查基本数据结构
            expect(body.data.status).toMatch(/^(healthy|unhealthy|degraded)$/);
            expect(body.data.timestamp).toBeTypeOf('string');
            expect(body.data.uptime).toBeTypeOf('number');
            expect(body.data.responseTime).toBeTypeOf('number');
            expect(body.data.version).toBeTypeOf('string');
            expect(body.data.environment).toBeTypeOf('string');
            expect(body.data.components).toBeTypeOf('object');
            
            // 检查组件状态
            if (body.data.components.mysql) {
                expect(body.data.components.mysql.status).toMatch(/^(healthy|unhealthy|degraded)$/);
            }
            if (body.data.components.redis) {
                expect(body.data.components.redis.status).toMatch(/^(healthy|unhealthy|degraded)$/);
            }
        });

        it('应该包含正确的时间戳格式', async () => {
            const res = await app.fetch(
                new Request('http://localhost/api/health', {
                    method: 'GET',
                }),
            );
            
            const body = (await res.json()) as any;
            
            // 验证ISO时间戳格式
            const timestamp = new Date(body.data.timestamp);
            expect(timestamp.toISOString()).toBe(body.data.timestamp);
        });

        it('应该返回合理的响应时间', async () => {
            const startTime = Date.now();
            const res = await app.fetch(
                new Request('http://localhost/api/health', {
                    method: 'GET',
                }),
            );
            const endTime = Date.now();
            
            const body = (await res.json()) as any;
            
            // 响应时间应该在合理范围内
            expect(body.data.responseTime).toBeGreaterThan(0);
            expect(body.data.responseTime).toBeLessThan(endTime - startTime + 100); // 允许一定误差
        });

        it('应该返回正确的环境信息', async () => {
            const res = await app.fetch(
                new Request('http://localhost/api/health', {
                    method: 'GET',
                }),
            );
            
            const body = (await res.json()) as any;
            
            expect(body.data.environment).toMatch(/^(development|production|test)$/);
            expect(body.data.uptime).toBeGreaterThan(0);
        });
    });

    describe('GET /api/health/detailed', () => {
        it('应该返回详细健康状态', async () => {
            const res = await app.fetch(
                new Request('http://localhost/api/health/detailed', {
                    method: 'GET',
                    headers: { 'Content-Type': 'application/json' },
                }),
            );
            
            const body = (await res.json()) as any;
            
            expect(res.status).toBe(200);
            expect(body.code).toBeTypeOf('number');
            expect(body.message).toBeTypeOf('string');
            expect(body.data).toBeTypeOf('object');
            
            // 检查基本健康检查数据
            expect(body.data.status).toMatch(/^(healthy|unhealthy|degraded)$/);
            expect(body.data.timestamp).toBeTypeOf('string');
            expect(body.data.uptime).toBeTypeOf('number');
            expect(body.data.responseTime).toBeTypeOf('number');
            expect(body.data.components).toBeTypeOf('object');
            
            // 检查详细信息
            if (body.data.system) {
                expect(body.data.system.platform).toBeTypeOf('string');
                expect(body.data.system.arch).toBeTypeOf('string');
                expect(body.data.system.nodeVersion).toBeTypeOf('string');
                expect(body.data.system.runtime).toBeTypeOf('string');
                expect(body.data.system.pid).toBeTypeOf('number');
                expect(body.data.system.cwd).toBeTypeOf('string');
            }
            
            if (body.data.performance) {
                expect(body.data.performance.cpuUsage).toBeTypeOf('object');
                expect(body.data.performance.memoryUsage).toBeTypeOf('object');
                expect(body.data.performance.uptime).toBeTypeOf('number');
                
                // 检查CPU使用情况
                expect(body.data.performance.cpuUsage.user).toBeTypeOf('number');
                expect(body.data.performance.cpuUsage.system).toBeTypeOf('number');
                
                // 检查内存使用情况
                expect(body.data.performance.memoryUsage.rss).toBeTypeOf('number');
                expect(body.data.performance.memoryUsage.heapTotal).toBeTypeOf('number');
                expect(body.data.performance.memoryUsage.heapUsed).toBeTypeOf('number');
                expect(body.data.performance.memoryUsage.external).toBeTypeOf('number');
                expect(body.data.performance.memoryUsage.arrayBuffers).toBeTypeOf('number');
            }
        });

        it('详细健康检查应该包含系统信息', async () => {
            const res = await app.fetch(
                new Request('http://localhost/api/health/detailed', {
                    method: 'GET',
                }),
            );
            
            const body = (await res.json()) as any;
            
            if (body.data.system) {
                expect(body.data.system.runtime).toBe('Bun');
                expect(body.data.system.pid).toBe(process.pid);
                expect(body.data.system.platform).toBe(process.platform);
                expect(body.data.system.arch).toBe(process.arch);
            }
        });
    });

    describe('健康检查依赖服务测试', () => {
        it('MySQL组件状态应该包含连接信息', async () => {
            const res = await app.fetch(
                new Request('http://localhost/api/health', {
                    method: 'GET',
                }),
            );
            
            const body = (await res.json()) as any;
            
            if (body.data.components.mysql) {
                expect(body.data.components.mysql.status).toMatch(/^(healthy|unhealthy|degraded)$/);
                
                if (body.data.components.mysql.details) {
                    expect(body.data.components.mysql.details.connection).toMatch(/^(active|failed)$/);
                    expect(body.data.components.mysql.details.host).toBeTypeOf('string');
                    expect(body.data.components.mysql.details.port).toBeTypeOf('string');
                }
                
                if (body.data.components.mysql.responseTime) {
                    expect(body.data.components.mysql.responseTime).toBeGreaterThan(0);
                }
            }
        });

        it('Redis组件状态应该包含连接信息', async () => {
            const res = await app.fetch(
                new Request('http://localhost/api/health', {
                    method: 'GET',
                }),
            );
            
            const body = (await res.json()) as any;
            
            if (body.data.components.redis) {
                expect(body.data.components.redis.status).toMatch(/^(healthy|unhealthy|degraded)$/);
                
                if (body.data.components.redis.details) {
                    expect(body.data.components.redis.details.connection).toMatch(/^(active|failed)$/);
                }
                
                if (body.data.components.redis.responseTime) {
                    expect(body.data.components.redis.responseTime).toBeGreaterThan(0);
                }
            }
        });
    });

    describe('健康检查错误处理', () => {
        it('健康检查应该处理组件异常', async () => {
            const res = await app.fetch(
                new Request('http://localhost/api/health', {
                    method: 'GET',
                }),
            );
            
            const body = (await res.json()) as any;
            
            // 即使有组件异常，也应该返回结构化的响应
            expect(res.status).toBe(200);
            expect(body.code).toBeTypeOf('number');
            expect(body.message).toBeTypeOf('string');
            expect(body.data).toBeTypeOf('object');
            
            // 如果有组件异常，整体状态可能是degraded或unhealthy
            if (body.data.status === 'unhealthy' || body.data.status === 'degraded') {
                // 应该有组件错误信息
                const components = body.data.components;
                let hasUnhealthyComponent = false;
                
                Object.values(components).forEach((component: any) => {
                    if (component && component.status === 'unhealthy') {
                        hasUnhealthyComponent = true;
                        expect(component.error).toBeTypeOf('string');
                    }
                });
                
                if (!hasUnhealthyComponent && body.data.error) {
                    expect(body.data.error).toBeTypeOf('string');
                }
            }
        });
    });

    describe('健康检查性能测试', () => {
        it('健康检查应该快速响应', async () => {
            const startTime = Date.now();
            
            const res = await app.fetch(
                new Request('http://localhost/api/health', {
                    method: 'GET',
                }),
            );
            
            const endTime = Date.now();
            const responseTime = endTime - startTime;
            
            expect(res.status).toBe(200);
            expect(responseTime).toBeLessThan(2000); // 应该在2秒内完成
        });

        it('详细健康检查应该在合理时间内完成', async () => {
            const startTime = Date.now();
            
            const res = await app.fetch(
                new Request('http://localhost/api/health/detailed', {
                    method: 'GET',
                }),
            );
            
            const endTime = Date.now();
            const responseTime = endTime - startTime;
            
            expect(res.status).toBe(200);
            expect(responseTime).toBeLessThan(3000); // 详细检查可能稍慢，但应该在3秒内完成
        });

        it('并发健康检查应该正常处理', async () => {
            const promises = [];
            const concurrentRequests = 5;
            
            for (let i = 0; i < concurrentRequests; i++) {
                promises.push(
                    app.fetch(
                                                 new Request('http://localhost/api/health', {
                             method: 'GET',
                         }),
                     ),
                 );
             }
            
            const responses = await Promise.all(promises);
            
            responses.forEach(res => {
                expect(res.status).toBe(200);
            });
        });
    });
}); 