/**
 * @file        Redis连接测试
 * @author      hotok
 * @date        2025-06-28
 * @lastEditor  hotok
 * @lastEditTime 2025-06-28
 * @description 测试Redis连接、健康检查、基本操作和状态管理功能
 */

import { describe, it, expect, beforeAll, afterAll, beforeEach } from 'vitest';
import { setTimeout } from 'node:timers';
import { Redis } from '@/utils/redis';
import { redisConfig } from '@/config/redis.config';

// 简单的延时函数
const delay = (ms: number) => new Promise(resolve => setTimeout(resolve, ms));

describe('Redis连接测试', () => {
    let redis: Redis;

    beforeAll(async () => {
        redis = new Redis();
        // 等待连接建立
        await delay(1000);
    });

    afterAll(async () => {
        if (redis) {
            await redis.disconnectRedis();
        }
    });

    beforeEach(async () => {
        // 确保每个测试前Redis连接正常
        if (!redis.redisClient.isOpen) {
            await redis.connectRedis();
        }
    });

    describe('Redis连接管理', () => {
        it('应该成功连接到Redis服务器', async () => {
            const isHealthy = await redis.checkRedisHealth();
            expect(isHealthy).toBe(true);
        });

        it('应该正确返回Redis连接状态', () => {
            const status = redis.getRedisStatus();
            expect(status).toEqual({
                isConnected: expect.any(Boolean),
                config: {
                    host: redisConfig.host,
                    port: redisConfig.port,
                    database: redisConfig.database,
                    connectName: redisConfig.connectName,
                },
            });
        });

        it('应该能够执行ping命令', async () => {
            const result = await redis.redisClient.ping();
            expect(result).toBe('PONG');
        });
    });

    describe('Redis基本操作', () => {
        const testKey = 'test_key';
        const testValue = 'test_value';

        it('应该能够设置和获取字符串值', async () => {
            // 设置值
            await redis.redisClient.set(testKey, testValue);
            
            // 获取值
            const result = await redis.redisClient.get(testKey);
            expect(result).toBe(testValue);
            
            // 清理测试数据
            await redis.redisClient.del(testKey);
        });

        it('应该能够设置带过期时间的值', async () => {
            const expiryTime = 2; // 2秒过期
            
            // 设置带过期时间的值
            await redis.redisClient.setEx(testKey, expiryTime, testValue);
            
            // 立即获取应该有值
            const result1 = await redis.redisClient.get(testKey);
            expect(result1).toBe(testValue);
            
            // 等待过期
            await delay(2100);
            
            // 过期后应该为null
            const result2 = await redis.redisClient.get(testKey);
            expect(result2).toBeNull();
        });

        it('应该能够检查键是否存在', async () => {
            // 设置测试键
            await redis.redisClient.set(testKey, testValue);
            
            // 检查存在
            const exists1 = await redis.redisClient.exists(testKey);
            expect(exists1).toBe(1);
            
            // 删除键
            await redis.redisClient.del(testKey);
            
            // 检查不存在
            const exists2 = await redis.redisClient.exists(testKey);
            expect(exists2).toBe(0);
        });

        it('应该能够删除键', async () => {
            // 设置测试键
            await redis.redisClient.set(testKey, testValue);
            
            // 删除键
            const deleteCount = await redis.redisClient.del(testKey);
            expect(deleteCount).toBe(1);
            
            // 验证键已被删除
            const result = await redis.redisClient.get(testKey);
            expect(result).toBeNull();
        });
    });

    describe('Redis Hash操作', () => {
        const hashKey = 'test_hash';
        const field1 = 'field1';
        const value1 = 'value1';
        const field2 = 'field2';
        const value2 = 'value2';

        it('应该能够设置和获取Hash字段', async () => {
            // 设置Hash字段
            await redis.redisClient.hSet(hashKey, field1, value1);
            
            // 获取Hash字段
            const result = await redis.redisClient.hGet(hashKey, field1);
            expect(result).toBe(value1);
            
            // 清理测试数据
            await redis.redisClient.del(hashKey);
        });

        it('应该能够设置和获取多个Hash字段', async () => {
            // 设置多个Hash字段
            await redis.redisClient.hSet(hashKey, field1, value1);
            await redis.redisClient.hSet(hashKey, field2, value2);
            
            // 获取所有Hash字段
            const result = await redis.redisClient.hGetAll(hashKey);
            expect(result).toEqual({
                [field1]: value1,
                [field2]: value2,
            });
            
            // 清理测试数据
            await redis.redisClient.del(hashKey);
        });
    });

    describe('Redis列表操作', () => {
        const listKey = 'test_list';
        const value1 = 'item1';
        const value2 = 'item2';

        it('应该能够推入和弹出列表元素', async () => {
            // 推入元素
            await redis.redisClient.lPush(listKey, value1);
            await redis.redisClient.lPush(listKey, value2);
            
            // 获取列表长度
            const length = await redis.redisClient.lLen(listKey);
            expect(length).toBe(2);
            
            // 弹出元素
            const poppedValue = await redis.redisClient.lPop(listKey);
            expect(poppedValue).toBe(value2);
            
            // 清理测试数据
            await redis.redisClient.del(listKey);
        });
    });

    describe('Redis连接错误处理', () => {
        it('健康检查在连接断开时应返回false', async () => {
            // 暂时断开连接
            await redis.disconnectRedis();
            
            // 健康检查应该返回false
            const isHealthy = await redis.checkRedisHealth();
            expect(isHealthy).toBe(false);
            
            // 重新连接
            await redis.connectRedis();
        });

        it('应该能够重新连接Redis', async () => {
            // 断开连接
            await redis.disconnectRedis();
            
            // 重新连接
            await redis.connectRedis();
            
            // 验证连接正常
            const isHealthy = await redis.checkRedisHealth();
            expect(isHealthy).toBe(true);
        });
    });

    describe('Redis性能测试', () => {
        it('应该能够快速执行大量set操作', async () => {
            const startTime = Date.now();
            const operations = [];
            
            // 执行100次set操作
            for (let i = 0; i < 1000; i++) {
                operations.push(redis.redisClient.set(`perf_test_${i}`, `value_${i}`));
            }
            
            await Promise.all(operations);
            const endTime = Date.now();
            
            expect(endTime - startTime).toBeLessThan(1000); // 应该在1秒内完成
            console.log(endTime - startTime);
            
            // 清理测试数据
            const deleteOperations = [];
            for (let i = 0; i < 100; i++) {
                deleteOperations.push(redis.redisClient.del(`perf_test_${i}`));
            }
            await Promise.all(deleteOperations);
        });
    });
}); 