/**
 * @file        邮件服务类型定义
 * @author      hotok
 * @date        2025-06-29
 * @lastEditor  hotok
 * @lastEditTime 2025-06-29
 * @description 定义邮件服务相关的类型，包括邮件发送、模板、配置等
 */

import type { Transporter } from 'nodemailer';

/**
 * 邮件传输器类型
 */
export type EmailTransporter = Transporter;

/**
 * 邮件优先级
 */
export type EmailPriority = 'high' | 'normal' | 'low';

/**
 * 邮件模板类型
 */
export type EmailTemplateType = 'activation' | 'passwordReset' | 'welcome' | 'notification' | 'passwordChanged';

/**
 * 邮件附件
 */
export interface EmailAttachment {
    /** 文件名 */
    filename: string;
    /** 文件内容 */
    content: string | Buffer;
    /** 内容类型 */
    contentType?: string;
    /** 内容编码 */
    encoding?: string;
    /** 内容ID */
    cid?: string;
    /** 内容配置 */
    contentDisposition?: 'attachment' | 'inline';
}

/**
 * 邮件发送选项
 */
export interface EmailSendOptions {
    /** 收件人邮箱数组 */
    to: string | string[];
    /** 抄送邮箱数组 */
    cc?: string | string[];
    /** 密送邮箱数组 */
    bcc?: string | string[];
    /** 邮件主题 */
    subject: string;
    /** 纯文本内容 */
    text?: string;
    /** HTML内容 */
    html?: string;
    /** 发件人 */
    from?: string;
    /** 回复邮箱 */
    replyTo?: string;
    /** 邮件优先级 */
    priority?: EmailPriority;
    /** 附件列表 */
    attachments?: EmailAttachment[];
    /** 邮件头信息 */
    headers?: Record<string, string>;
    /** 消息ID */
    messageId?: string;
    /** 引用消息ID */
    references?: string;
    /** 回复消息ID */
    inReplyTo?: string;
}

/**
 * 邮件模板参数
 */
export interface EmailTemplateParams {
    /** 用户名 */
    username?: string;
    /** 用户昵称 */
    nickname?: string;
    /** 用户邮箱 */
    email?: string;
    /** 激活链接 */
    activationUrl?: string;
    /** 激活码 */
    activationCode?: string;
    /** 重置链接 */
    resetUrl?: string;
    /** 重置码 */
    resetCode?: string;
    /** 过期时间 */
    expireTime?: string;
    /** 系统名称 */
    systemName?: string;
    /** 公司名称 */
    companyName?: string;
    /** 支持邮箱 */
    supportEmail?: string;
    /** 其他自定义参数 */
    [key: string]: any;
}

/**
 * 邮件模板发送选项
 */
export interface EmailTemplateSendOptions {
    /** 收件人邮箱数组 */
    to: string | string[];
    /** 模板类型 */
    template: EmailTemplateType;
    /** 模板参数 */
    params: EmailTemplateParams;
    /** 抄送邮箱数组 */
    cc?: string | string[];
    /** 密送邮箱数组 */
    bcc?: string | string[];
    /** 自定义主题（覆盖模板默认主题） */
    subject?: string;
    /** 邮件优先级 */
    priority?: EmailPriority;
    /** 附件列表 */
    attachments?: EmailAttachment[];
    /** 语言标识 */
    locale?: string;
}

/**
 * 邮件发送结果
 */
export interface EmailSendResult {
    /** 是否发送成功 */
    success: boolean;
    /** 消息ID */
    messageId?: string;
    /** 发送的邮箱数量 */
    accepted: string[];
    /** 被拒绝的邮箱 */
    rejected: string[];
    /** 待处理的邮箱 */
    pending: string[];
    /** 响应信息 */
    response?: string;
    /** 错误信息 */
    error?: string;
    /** 发送时间 */
    sentAt: Date;
    /** 重试次数 */
    retryCount?: number;
}

/**
 * 邮件队列任务
 */
export interface EmailQueueTask {
    /** 任务ID */
    id: string;
    /** 邮件选项 */
    options: EmailSendOptions | EmailTemplateSendOptions;
    /** 创建时间 */
    createdAt: Date;
    /** 计划发送时间 */
    scheduledAt?: Date;
    /** 重试次数 */
    retryCount: number;
    /** 最大重试次数 */
    maxRetries: number;
    /** 优先级 */
    priority: number;
    /** 任务状态 */
    status: 'pending' | 'processing' | 'completed' | 'failed';
    /** 错误信息 */
    error?: string;
}

/**
 * 邮件服务配置
 */
export interface EmailServiceConfig {
    /** SMTP配置 */
    smtp: {
        host: string;
        port: number;
        secure: boolean;
        auth: {
            user: string;
            pass: string;
        };
        connectionTimeout: number;
        greetingTimeout: number;
        socketTimeout: number;
    };
    /** 邮件基础配置 */
    email: {
        from: string;
        fromName: string;
        replyTo: string;
        charset: string;
        priority: EmailPriority;
    };
    /** 发送选项 */
    options: {
        retryAttempts: number;
        retryDelay: number;
        enableQueue: boolean;
        maxRecipients: number;
        rateLimit: number;
    };
}

/**
 * 邮件服务状态
 */
export interface EmailServiceStatus {
    /** 服务状态 */
    status: 'healthy' | 'unhealthy' | 'degraded';
    /** 传输器状态 */
    transporterStatus: 'connected' | 'disconnected' | 'error';
    /** 队列状态 */
    queueStatus?: {
        pending: number;
        processing: number;
        failed: number;
    };
    /** 错误信息 */
    error?: string;
    /** 最后检查时间 */
    lastCheckAt: Date;
}

/**
 * 扩展Elysia Context，添加邮件服务实例
 */
export interface EmailContext {
    /** 邮件服务实例 */
    emailService: any; // 这里会在service创建后更新为具体类型
}

/**
 * 邮件健康检查结果
 */
export interface EmailHealthCheckResult {
    /** 状态 */
    status: 'healthy' | 'unhealthy';
    /** 响应时间 */
    responseTime: number;
    /** 服务状态 */
    serviceStatus: EmailServiceStatus;
    /** 错误信息 */
    error?: string;
} 