/**
 * @file        错误类型定义
 * @author      hotok
 * @date        2025-06-28
 * @lastEditor  hotok
 * @lastEditTime 2025-06-28
 * @description 自定义业务异常类型和错误响应类型定义
 */

/**
 * 错误响应结构
 */
export interface ErrorResponse {
    /** 错误码 */
    code: number;
    /** 错误消息 */
    message: string;
    /** 返回数据，错误时通常为null */
    data: null;
    /** 请求追踪ID */
    traceId?: string;
    /** 错误详情（仅开发环境） */
    details?: any;
    /** 堆栈信息（仅开发环境） */
    stack?: string;
}

/**
 * 业务异常类型枚举
 */
export enum BusinessErrorType {
    /** 参数校验错误 */
    VALIDATION_ERROR = 'VALIDATION_ERROR',
    /** 认证错误 */
    AUTH_ERROR = 'AUTH_ERROR',
    /** 权限错误 */
    PERMISSION_ERROR = 'PERMISSION_ERROR',
    /** 业务逻辑错误 */
    BUSINESS_ERROR = 'BUSINESS_ERROR',
    /** 资源不存在 */
    NOT_FOUND_ERROR = 'NOT_FOUND_ERROR',
    /** 系统错误 */
    SYSTEM_ERROR = 'SYSTEM_ERROR',
    /** 外部服务错误 */
    EXTERNAL_ERROR = 'EXTERNAL_ERROR',
}

/**
 * 业务异常类
 */
export class BusinessError extends Error {
    /** 错误码 */
    public readonly code: number;
    /** 错误类型 */
    public readonly type: BusinessErrorType;
    /** 额外数据 */
    public readonly data?: any;
    /** HTTP状态码 */
    public readonly statusCode: number;

    constructor(type: BusinessErrorType, message: string, code: number, statusCode: number = 400, data?: any) {
        super(message);
        this.name = 'BusinessError';
        this.type = type;
        this.code = code;
        this.statusCode = statusCode;
        this.data = data;

        // 保持正确的堆栈跟踪
        if (Error.captureStackTrace) {
            Error.captureStackTrace(this, BusinessError);
        }
    }
}

/**
 * 常用业务异常工厂类
 */
export class BusinessErrors {
    /** 参数校验错误 */
    static validation(message: string, data?: any): BusinessError {
        return new BusinessError(BusinessErrorType.VALIDATION_ERROR, message, 400, 400, data);
    }

    /** 认证错误 */
    static auth(message: string = '认证失败'): BusinessError {
        return new BusinessError(BusinessErrorType.AUTH_ERROR, message, 401, 401);
    }

    /** 权限错误 */
    static permission(message: string = '权限不足'): BusinessError {
        return new BusinessError(BusinessErrorType.PERMISSION_ERROR, message, 403, 403);
    }

    /** 资源不存在 */
    static notFound(message: string = '资源不存在'): BusinessError {
        return new BusinessError(BusinessErrorType.NOT_FOUND_ERROR, message, 404, 404);
    }

    /** 业务逻辑错误 */
    static business(message: string, code: number = 1000): BusinessError {
        return new BusinessError(BusinessErrorType.BUSINESS_ERROR, message, code, 400);
    }

    /** 系统错误 */
    static system(message: string = '系统错误'): BusinessError {
        return new BusinessError(BusinessErrorType.SYSTEM_ERROR, message, 500, 500);
    }

    /** 外部服务错误 */
    static external(message: string = '外部服务错误'): BusinessError {
        return new BusinessError(BusinessErrorType.EXTERNAL_ERROR, message, 502, 502);
    }
}
