/**
 * @file        JWT类型定义
 * @author      hotok
 * @date        2025-06-29
 * @lastEditor  hotok
 * @lastEditTime 2025-06-29
 * @description JWT Token载荷和用户信息的TypeScript类型定义
 */

/**
 * JWT Token中的用户信息类型
 * @description 存储在JWT Token中的用户基本信息，不包含敏感数据
 */
export interface JwtUserType {
    /** 用户ID */
    userId: number;
    /** 用户名 */
    username: string;
    /** 用户邮箱 */
    email: string;
    /** 用户昵称 */
    nickname?: string;
    /** 用户状态：0-禁用，1-启用 */
    status: number;
    /** 用户角色（可选，用于权限控制） */
    role?: string;
}

/**
 * 完整的JWT载荷类型
 * @description JWT Token的完整载荷，包含用户信息和JWT标准字段
 */
export interface JwtPayloadType extends JwtUserType {
    /** Token发行时间（秒级时间戳） */
    iat: number;
    /** Token过期时间（秒级时间戳） */
    exp: number;
    /** Token主题，通常是用户ID */
    sub?: string;
    /** Token发行者 */
    iss?: string;
    /** Token受众 */
    aud?: string;
    /** JWT ID */
    jti?: string;
    /** Token生效时间（秒级时间戳） */
    nbf?: number;
}

/**
 * JWT认证上下文类型
 * @description 在认证中间件中使用的用户上下文类型
 */
export interface JwtContextType {
    /** 当前认证用户信息 */
    user: JwtUserType;
    /** 原始JWT载荷 */
    payload?: JwtPayloadType;
}

/**
 * JWT生成参数类型
 * @description 生成JWT Token时的参数类型
 */
export interface JwtSignOptionsType {
    /** 用户信息 */
    user: JwtUserType;
    /** 自定义过期时间（可选） */
    expiresIn?: string;
    /** 自定义发行者（可选） */
    issuer?: string;
    /** 自定义受众（可选） */
    audience?: string;
} 