/**
 * @file        JWT辅助工具
 * @author      hotok
 * @date        2025-06-29
 * @lastEditor  hotok
 * @lastEditTime 2025-06-29
 * @description 提供JWT相关的辅助函数，用于Controller中的JWT操作
 */

import type { JwtUserType, JwtPayloadType } from '@/type/jwt.type';
import type { UserInfoType } from '@/modules/example/example.schema';

/**
 * 从完整用户信息创建JWT用户信息
 * @param userInfo 完整的用户信息
 * @param role 可选的用户角色
 * @returns JwtUserType JWT中的用户信息
 * @modification hotok 2025-06-29 创建JWT用户信息转换函数
 */
export function createJwtUser(userInfo: UserInfoType, role?: string): JwtUserType {
    return {
        userId: userInfo.id,
        username: userInfo.username,
        email: userInfo.email,
        nickname: userInfo.nickname,
        status: userInfo.status,
        role: role,
    };
}

/**
 * 创建JWT载荷
 * @param userInfo 完整的用户信息
 * @param options 可选配置
 * @returns JWT载荷对象（不包含iat、exp等自动生成字段）
 * @modification hotok 2025-06-29 创建JWT载荷生成函数
 */
export function createJwtPayload(
    userInfo: UserInfoType, 
    options?: {
        role?: string;
        issuer?: string;
        audience?: string;
        subject?: string;
    },
): Omit<JwtPayloadType, 'iat' | 'exp'> {
    const jwtUser = createJwtUser(userInfo, options?.role);
    
    return {
        ...jwtUser,
        sub: options?.subject || userInfo.id.toString(),
        iss: options?.issuer || 'elysia-api',
        aud: options?.audience || 'web-client',
    };
}

/**
 * 验证JWT载荷中的用户是否有效
 * @param payload JWT载荷
 * @returns boolean 用户是否有效
 * @modification hotok 2025-06-29 添加用户有效性验证
 */
export function isValidJwtUser(payload: JwtPayloadType): boolean {
    // 检查必需字段
    if (!payload.userId || !payload.username || !payload.email) {
        return false;
    }
    
    // 检查用户状态（1为启用）
    if (payload.status !== 1) {
        return false;
    }
    
    return true;
}

/**
 * 检查JWT是否即将过期
 * @param payload JWT载荷
 * @param thresholdMinutes 阈值分钟数（默认30分钟）
 * @returns boolean 是否即将过期
 * @modification hotok 2025-06-29 添加过期检查功能
 */
export function isTokenExpiringSoon(payload: JwtPayloadType, thresholdMinutes: number = 30): boolean {
    if (!payload.exp) return false;
    
    const now = Math.floor(Date.now() / 1000);
    const threshold = thresholdMinutes * 60;
    
    return (payload.exp - now) <= threshold;
}

/**
 * 获取Token剩余有效时间
 * @param payload JWT载荷
 * @returns number 剩余秒数，-1表示已过期或无过期时间
 * @modification hotok 2025-06-29 添加时间计算功能
 */
export function getTokenRemainingTime(payload: JwtPayloadType): number {
    if (!payload.exp) return -1;
    
    const now = Math.floor(Date.now() / 1000);
    const remaining = payload.exp - now;
    
    return remaining > 0 ? remaining : -1;
}

/**
 * 格式化Token剩余时间为可读字符串
 * @param seconds 剩余秒数
 * @returns string 格式化的时间字符串
 * @modification hotok 2025-06-29 添加时间格式化功能
 */
export function formatRemainingTime(seconds: number): string {
    if (seconds <= 0) return '已过期';
    
    const hours = Math.floor(seconds / 3600);
    const minutes = Math.floor((seconds % 3600) / 60);
    const remainingSeconds = seconds % 60;
    
    if (hours > 0) {
        return `${hours}小时${minutes}分钟`;
    } else if (minutes > 0) {
        return `${minutes}分钟${remainingSeconds}秒`;
    } else {
        return `${remainingSeconds}秒`;
    }
}

/**
 * JWT用户信息的简化版本（用于日志记录）
 * @param user JWT用户信息
 * @returns string 简化的用户信息字符串
 * @modification hotok 2025-06-29 添加用户信息格式化功能
 */
export function formatUserForLog(user: JwtUserType): string {
    return `用户ID:${user.userId} 用户名:${user.username} 状态:${user.status === 1 ? '启用' : '禁用'}`;
} 